<?php
/*
This code is Copyright(c) 2019 by ubergeek under the GPL 3 or later.
Parsedown is licensed under the MIT license.
*/

include('config.php');
include('parsedown-1.7.3/Parsedown.php');

$page = isset($_GET['page']) ? $_GET['page'] : 'main';

// Sanitize page request so we don't allow to read EVERY markdown file
// for example ../../../home/foobar/mysecretdocument
if (strpos($page, "../") !== false) {
    header('HTTP/1.0 403 Forbidden');
    exit();
}

// Strip leading/trailing '/' from $page
$page = trim($page, '/');

// Check whether requested page is file or folder
$path = "articles/$page";
if (is_dir($path)) {
    // The requested page is a directory
    $index_path = "$path/main.md";
    if (is_file($index_path)) {
        // There is an index page (main.md), don't list children automatically
        $dir = false;
        $content_file = "$index_path";
        $content = file_get_contents($content_file);
    } else {
        // No index page (main.md) so we'll use $dir_entries to list entries
        $dir = true;
        $dir_entries = scandir($path);
        if ($dir_entries === false) {
            echo "ERROR: Failed to read from folder.";
            exit();
        }
    }
} else {
    // The requested page is not a directory
    // Try to find a corresponding markdown file or 404
    $dir = false;
    $content_file = "$path.md";
    // TODO: here wiki does not return an actual 404 header status code
    $content = file_exists($content_file) ? file_get_contents($content_file) : str_replace('$page', "$page", file_get_contents("includes/404.md"));
}

// When you need some debugging
//echo "<br>page: $page";
//echo "<br>content file: $content_file";

$Parsedown = new Parsedown();
$Parsedown->setSafeMode(true);

if(isset($_GET['style']))
	$site_style = $_GET['style'];
else
	$site_style="site";

$header  = file_get_contents("includes/header.md");
$sidebar = file_get_contents("includes/sidebar.md");
$footer  = file_get_contents("includes/footer.md");

// TODO: Stylesheet URL assumes wiki is not operate in subfolder

print "<!DOCTYPE html>
<html lang='en'>
	<head>
		<title>$site_name - $page</title>
		<link rel='stylesheet' type='text/css' href='/includes/$site_style.css'>
	</head>
	<body>
<!-- Begin Header -->

	<div id='header'>";

print $Parsedown->text($header);

print "
		</div>
<!-- End Header -->
";

print "<hr>
	<div id='body'>

<!-- Begin Sidebar  -->
		<div id='sidebar'>
";

echo $Parsedown->text($sidebar);

print "		</div>
<!-- End Sidebar -->

<!-- Begin Body -->
		<div id='content'>";

if ($dir) {
    // List entries in $dir_entries
    echo "<h1>$page</h1>";
    echo '<ul>';
    // Register a bool to know if we actually printed something
    $found_children = false;
    foreach($dir_entries as $entry) {
        if ($entry[0] === '.') {
            // Skip hidden files/folders
            continue;
        }
        // Check whether the entry is a subfolder of the current page ($path on disk)
        if (is_dir("$path/$entry")) {
            $found_children = true;
            echo "<li><a href=\"$entry/\">$entry/</a></li>";
        } else {
            // Entry is a file, check whether it's a markdown file
            // if (!str_ends_with("$entry", ".md")) { // PHP8!
            if (preg_match('/\.md$/', $entry) === 0) {
                continue;
            }
            $found_children = true;
            // strip '.md' extension from it
            $entry = rtrim($entry, ".md");
            echo "<li><a href=\"$entry\">$entry</a></li>";
        }
    }
    echo '</ul>';

    if ($found_children === false) {
        echo "<p>No pages yet in this section.</p>";
    }
} else {
    // Parse requested page markdown
    echo $Parsedown->text($content);
}

print "		</div>
<!-- End Body -->

	</div>

<!-- Begin Footer -->
	<div id='footer'>
	<hr>
";

echo $Parsedown->text($footer);

print "	</div>
<!-- End Footer -->

	</body>
</html>";
?>
